/*
 *  batch [-Pn] [-v] [-d] [-j jobname] [-m mailuserid] [-u userid]
 *        queue [file ...] [-c "command"]
 */
#include "bat_system.h"		/* sets BSD4_2 and JOBCTL flags */

#include <stdio.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>
#include <pwd.h>
#include <syslog.h>

#include "bat_common.h"

#ifdef BSD4_2
#define	STATIC	static
#else
#define	STATIC				/* avoid Iris compiler bug */
#endif

extern char *getenv();
extern char *index();
extern char *getwd();
extern int errno;

static char grade = 'q';	/* Priority of this job */
static int vflag;
static enum { BOURNE, CSH } shelltype;
static char *deleteerr;

/* Make sure all this matches stuff in batchd too.
 */
static char *spooldir = SPOOLDIR;
static char *pidfile = PIDFILE;
static char errstr[STRSIZ];	/* Buffer for SPRINTF error messages */
static int debug = 0;		/* turn on for debugging output */

main(argc, argv, arge)
	char **argv;
	char **arge;
{
	register int i;
	register char **ep;
	int currumask;
	char *jobname = NULL;
	char *userid = NULL;
	char *mailuserid = NULL;
	char *queue = NULL;
	int nocshrc = 0;		/* turn on to enable "#!/bin/csh -f" */
	char fname[MAXPATHLEN];
	char tfname[MAXPATHLEN];
	char cfname[MAXPATHLEN];
	long spoolnumber;
	char *shell;
	FILE *tf;
	char cwdirectory[MAXPATHLEN];
	struct stat sbuf;
	char *command = NULL;
	int uid;
	extern long unique();
	extern void cleanup();
	extern int optind;
	extern char *optarg;

	uid = getuid();
	currumask = umask(0077);	/* Privacy for created files */

#ifndef DEBUG
	if(geteuid() != 0) {
		syserror1("Effective uid is %d, not root (0)\n",
			geteuid());
		cleanup();
	}
#endif

	/*
	 * Parse command line.
	 */
	while ((i = getopt(argc, argv, "P:d:p:fj:m:u:vD")) != EOF)
		switch (i) {
		case 'P':
			grade = *optarg;
			break;
		case 'd':
			if (uid)
				Usage();	/* security */
			spooldir = optarg;
			break;
		case 'D':
			debug = 1;
			break;
		case 'f':
			nocshrc = 1;	/* use "#!/bin/csh -f" */
			break;
		case 'j':
			if (jobname)
				Usage();
			jobname = optarg;
			break;
		case 'm':
			if (mailuserid)
				Usage();
			mailuserid = optarg;
			break;
		case 'p':
			if (uid)
				Usage();	/* security */
			pidfile = optarg;
			break;
		case 'u':
			if (userid || uid != 0)
				Usage();
			userid = optarg;
			break;
		case 'v':
			vflag = 1;
			break;
		default:
			Usage();
		}
	if (optind >= argc)
		Usage();
	queue = argv[optind++];

	if (optind+2 == argc) {	/* Handle -c "command" as arguments */
		if (strcmp(argv[optind],"-c") == 0) {
			optind++;
			command = argv[optind++];
		}
	}

	/*
	 * at this point, argv[optind .. argc-1] are the (optional) input files.
	 * Pick up the first non-stdin file name and make it the job name.
	 */
	if( jobname == NULL ){
		for( i = optind; i < argc ; i++) {
			if(argv[i][0] == '-' && argv[i][1] == '\0')
				continue;
			jobname = argv[i];
		}
	}

	/*
	 *  The grade is a-z and only SU can set it to values lower than 'h'.
	 */
	if(grade < 'a'  ||  grade > 'z'  ||  (grade < 'h')  &&  uid) {
		error1("Grade `%c' is invalid.\n", grade);
		cleanup();
	}

	/*
	 * Have to check for queue names like:  .  ..  ../../..
	 */
	if( index(queue,'/') != NULL
	    || strcmp(queue,".") == 0 || strcmp(queue,"..") == 0 ){
		error1("'%s': Invalid queue name\n", queue);
		cleanup();
	}
	if( strlen(queue) >= DIRSIZ ){
		error2("'%s': Queue name longer than %d characters\n",
			queue, DIRSIZ-1);
		cleanup();
	}

	/*
	 * Print "no such queue" if the queue is not there or if
	 * it is not a directory.
	 */
	(void)sprintf(fname, "%s/%s", spooldir, queue);
	if(stat(fname, &sbuf) < 0  ||
	   (errno=ENOENT, (sbuf.st_mode & S_IFMT) != S_IFDIR)) {
		if(errno == ENOENT) {
			error1("'%s': No such queue\n", queue);
		} else {
			perror(fname);
			syserror1("'%s': Can't stat\n", fname);
		}
		cleanup();
	}

	/*
	 * If the underlying Q_CFDIR is missing, create it.
	 */
	(void)sprintf(fname, "%s/%s/%s", spooldir, queue, Q_CFDIR);
	while( stat(fname,&sbuf) < 0 ||
	  (errno=ENOTDIR, (sbuf.st_mode & S_IFMT) != S_IFDIR)) {
		int saveumask;

		if( errno != ENOENT ){
			perror(fname);
			syserror1("'%s': Cannot stat queue directory\n", fname);
			cleanup();
		}
		saveumask = umask(022);
		if( mkdir(fname, 0775) < 0 ){
			perror(fname);
			syserror1("Cannot mkdir(%s)\n", fname);
			umask(saveumask);
			cleanup();
		}
		umask(saveumask);
		if( debug )
			fprintf(stderr, "SYSLOG: batch: created directory '%s'\n", fname);
		else
			syslog(LOG_INFO, "batch: created directory '%s'\n", fname);
		/* mkdir worked; loop back to try stat() again */
	}

	/*
	 * Spool temp file name starts with tf, followed by grade, followed by
	 * a unique sequential number.  Real file name starts with cf.
	 * The files are sorted by this name for priority of execution.
	 */
#ifdef BSD4_2
#define FMT "%s/%s/%s/%cf%c%ld"
#else
#define FMT "%s/%s/%s/%cf%c%010D"
#endif
	spoolnumber = unique();
	(void)sprintf(tfname, FMT, spooldir, queue,
		Q_CFDIR, 't', grade, spoolnumber);
	(void)sprintf(cfname, FMT, spooldir, queue,
		Q_CFDIR, 'c', grade, spoolnumber);

	{
	    static char sigs[] = { SIGINT, SIGHUP, SIGTERM, SIGQUIT };

	    for (i = 0; i < sizeof sigs/sizeof sigs[0]; i++)
		    if (signal(sigs[i], SIG_IGN) != SIG_IGN)
			    signal(sigs[i], cleanup);
	}

	deleteerr = tfname;		/* cleanup() deletes this file */
#ifdef DEBUG
	tf = stdout;
#else
	if((tf = fopen(tfname, "w")) == NULL) {
		perror(tfname);
		syserror1("'%s': Error opening temp file\n", tfname);
		cleanup();
	}
#endif

	/*
	 * Set up a $USER mailuserid for ROOT if one isn't given.
	 */
	if( uid == 0 && mailuserid == NULL
	  && (mailuserid = getenv("USER")) != NULL ){
		if( getpwnam(mailuserid) == NULL ){
			error1("Can't find $USER='%s' in passwd file\n",
				mailuserid);
			cleanup();
		}
	}
	if(getwd(cwdirectory) == NULL){
		error1("Can't get current directory: %s\n", cwdirectory);
		cleanup();
	}

	/*
	 *  "shelltype" tells which type of environment setting to use.
	 */
	if((shell = getenv("SHELL")) == NULL)
		shell = BAT_SH;
	if(strcmp(shell, BAT_SH) == 0)
		shelltype = BOURNE;
	else if(strncmp(shell+strlen(shell)-3, BAT_CSHELL, 3) == 0)
		shelltype = CSH;
	else {
		/* Assume it's a bourne-like shell. Don't you just
		   love hard-coded shells? */
		shelltype = BOURNE;
		/*error1("'%s': Unrecognized shell in $SHELL variable\n", shell);*/
		/*cleanup();*/
	}

	/* Write out the shell line and the header information that
	 * the Batch Daemon will need to start running this job.
	 * Since it will be directly executable by the system put out a
	 * system magic word followed by the current shell.
	 *
	 * Things to associate with the queued batch file:
	 *
	 * <no name>: (protected)
	 *   - local userid on this machine for executing/accounting
	 *      -> this is the uid of the queued batch file
	 * jobname: (user-settable)
	 *   - job name/identifier
	 * userid: (protected)
	 *   - full userid@host responsible for job
	 *      - same as local userid, unless file is uid ROOT
	 *      - for uid ROOT file, believe the "userid" header, if any
	 * mailuserid: (user-settable)
	 *   - userid@host for mail from job
	 *      - use "userid" or local uid if no "mailuserid" found
	 * directory: (semi-protected)
	 *   - current directory
	 * umask: (user-settable)
	 *   - current umask
	 * ???: (user-settable)
	 *   - shell to run it with
	 * ???: (protected)
	 *   - priority of file
	 * ???: (semi-protected)
	 *   - system resource limits
	 * ???: (user-settable)
	 *   - environment
	 *     -> currently copied as "setenv" commands to the batch file
	 */
	fprintf(tf, "#!%s %s\n", shell, (shelltype==CSH && nocshrc)?"-f":"");
	if( jobname )
		fprintf(tf, "# jobname: %s\n", jobname);
	if( userid )
		fprintf(tf, "# userid: %s\n", userid );
	if( mailuserid )
		fprintf(tf, "# mailuserid: %s\n", mailuserid );
	fprintf(tf, "# umask: 0%03o\n", currumask );
	/*
	 * Peel off the leading /tmp/mnt_ from the cwd,
	 * so things work correctly when using Sun's automounter.
	 *	- Dr. J. Holzfuss
	 */
	fprintf(tf, "# directory: %s\n",
		strncmp(cwdirectory, "/tmp_mnt/", 9) == 0 ?
		cwdirectory+8 : cwdirectory);
	fprintf(tf, "\n");	/* null line ends header */

	/* The big problem with using these commands is that C Shell users
	 * can redefine them with aliases in their .cshrc files.
	 * That can screw up system programs, such as typeset, that
	 * try to create batch jobs for users.  So I added the -f
	 * "don't use .cshrc" option.  -IAN!
	 */
	if( shelltype == CSH )
		fprintf(tf,"set histchars\n");

	/*
	 * Set everything up so that it will execute in the same env. as
	 * we have right now.  Dump the current environment variables. 
	 * Ignore them if they don't have = signs, throw away TERMCAP and TERM.
	 */
	for(ep = arge ; *ep ; ep++) {
		register char *p;

		if((p = index(*ep, '=')) == NULL)
			continue;
		*p++ = 0;
		if(strcmp(*ep, "TERMCAP") == 0)
			continue;
		if(strcmp(*ep, "TERM") == 0)
			continue;
		putenv(tf, *ep, p);
	}

	/*
	 * If using csh, set $home so the ~ character expands correctly.
	 * This should be something CSH does automatically when you
	 * set $HOME.
	 */
	if(shelltype == CSH)
		fprintf(tf, "if ( $?HOME ) then\n\tset home=$HOME\nendif\n");
	/*
	 *  Dump the files specified; or stdin otherwise.
	 *  First, put out a CONTROL_STR comment line to indicate we
	 *  are starting the real part of the command file.
	 */
	fprintf(tf,CONTROL_STR);
	for( i = optind; i < argc ; i++) {
		register FILE *input;
		register char *name;

 		if(argv[i][0] == '-' && argv[i][1] == '\0'){
			input = stdin;
			name  = "(stdin)";
		}
		else{
			/*
			 * Verify read permission using access() for real uid.
			 */
			if (access(name=argv[i], R_OK) != 0 ||
				(input = fopen(argv[i], "r")) == NULL) {
				perror(argv[i]);
				error1("'%s': Can't be opened for reading\n", argv[i]);
				cleanup();
			}
		}
		concat(tf, input, name);
		if( input != stdin )
			(void)fclose(input);
		else
			clearerr(input);
	}
	/*
	 * If no input file or -c used, then use stdin.
	 */
	if (command || optind >= argc) {
		if (command)
			fprintf(tf, "%s\n", command);
		else
			concat(tf, stdin, "(stdin)");
	}
#ifdef DEBUG
	exit(0);
#endif
	/*
	 *  Set everything up for direct execution; and then link it to
	 *  its real filename.
	 */
	if (chown(tfname, uid, getgid()) < 0) {
		perror(tfname);
		syserror1("'%s': chown failed\n", tfname );
		cleanup();
	}
	if (chmod(tfname, 0700) < 0) {
		perror(tfname);
		syserror1("'%s': chmod failed\n", tfname);
		cleanup();
	}
	if (fclose(tf) == EOF) {
		perror(tfname);
		syserror1("'%s': FCLOSE failed\n", tfname);
		cleanup();
	}
#ifdef BSD4_2
	if (rename(tfname, cfname) < 0)
#else /*BSD4_2*/
	if(link(tfname, cfname) < 0 || unlink(tfname) < 0)
#endif /*BSD4_2*/
	{
		perror(tfname);
		syserror2("'%s': Rename to '%s' failed\n", tfname, cfname);
		cleanup();
	}
	/* Wake up the BATCH Daemon */
	if ((tf = fopen(pidfile, "r")) == NULL){
		syswarn1("'%s': missing; BATCH Daemon may not be running\n",
			pidfile);
	}
	else {
		int pid;

		if( fscanf(tf, "%d", &pid) != 1 || pid <= 1 ){
			syswarn1("'%s': BATCH Daemon lock file corrupted\n",
				pidfile );
		}
		else {
			if( kill(pid, SIGALRM) < 0 ){
				switch( errno ){
				case ESRCH:
					syswarn2("'%s': BATCH Daemon process id %d does not exist\n",
						pidfile, pid );
					break;
				default:
					perror("BATCH Daemon Process");
					syswarn2("'%s': KILL BATCH Daemon process id %d failed\n",
						pidfile, pid );
					break;
				}
			}
		}
		(void) fclose(tf);
	}
	if(vflag)
		printf("%s\n", cfname);
	return(0);
}

/*
 * User error.
 */
	STATIC
error(s)
	char *s;
{
	fprintf(stderr, "batch: %s", s);
	if( index(s,'\n') == NULL )
		fprintf(stderr, "\n");
	fprintf(stderr, "batch: No job submitted.\n");
}

/*
 * System error; job not submitted.
 */
	STATIC
syserror(s)
	char *s;
{
	fprintf(stderr, "batch: %s", s);
	if( index(s,'\n') == NULL )
		fprintf(stderr, "\n");
	fprintf(stderr, "batch: No job submitted.\n");
	if( debug )
		fprintf(stderr, "SYSLOG: batch: %s\n", s);
	else
		syslog(LOG_ALERT, "batch: %s", s);
}

/*
 * System warning.
 */
	STATIC
syswarn(s)
	char *s;
{
	fprintf(stderr, "batch: Warning -- %s", s);
	if( index(s,'\n') == NULL )
		fprintf(stderr, "\n");
	if( debug )
		fprintf(stderr, "SYSLOG: batch: %s\n", s);
	else
		syslog(LOG_ALERT, "batch: %s", s);
}

	STATIC void
cleanup()
{
	if(deleteerr != NULL)
		(void)unlink(deleteerr);
	exit(1);
}

	STATIC
concat(to, from, file)
	register FILE *to, *from;
	char *file;
{
	register int c;
	register int ttyin;

	ttyin = isatty(fileno(from));

	if( ttyin ){
		fprintf( stderr, "batch> ");
		fflush(stderr);
	}
	while ((c = getc(from)) != EOF){
		putc(c, to);
		if( c == '\n' && ttyin ){
			fprintf( stderr, "batch> ");
			fflush(stderr);
		}
	}
	/* errors are checked when we fclose(to) */
}

/*
 *  Dump out an environment variable in the format for this shell.
 */
	STATIC
putenv(tf, ep, p)
	FILE *tf;
	char *ep, *p;
{
	switch(shelltype) {
	case BOURNE:
		fprintf(tf, "%s=", ep);
		putstring(tf, p );
		fprintf(tf, "; export %s\n", ep);
		break;
	case CSH:
		fprintf(tf, "setenv %s ", ep);
		putstring(tf, p );
		fprintf(tf, "\n");
		break;
	}
}

/*
 *  God help us; put out a string with whatever escapes are needed to make
 *  damn sure that the shell doesn't interpret it.  BOURNE shell doesn't
 *  need escapes on newlines; CSH does.
 */
	STATIC
putstring(tf, s)
	register FILE *tf;
	register char *s;
{
	putc('\'', tf);
	while(*s) {
		if(*s == '\n' && shelltype == CSH )
			putc('\\', tf);
		if(*s == '\'')
			s++, fprintf(tf,"'\"'\"'"); /* 'You can'"'"'t come' */
		else
			putc(*s++, tf);
	}
	putc('\'', tf);
}

	STATIC
Usage()
{
	error("Usage: batch [-Pn] [-v] [-d] [-j jobname] [-m mailuserid] [-u userid] queue [file ...] [-c \"command\"]\n");
	cleanup();
}

/*
 * Generate a unique number.
 * We use the time of day for the high bits
 * (so sorting the files by ASCII string compares
 * sorts them by creation time - an important property)
 * concatenated with some lower bits of the process ID.
 *
 * Possible screwups:
 *    -	If this UNIX ever creates more than
 *	2^PIDBITS processes in one second, and two of them are
 *	"batch", then they might come up with the same filename.
 *	Make PIDBITS large enough to make this arbitrarily unlikely.
 *    - Every 24855/(2^PIDBITS) days, the unique number overflows.
 *	This results in newer entries being processed before ones
 *	that predate the rollover.  This is a problem only if
 *	the queue never drains completely, leaving the old ones stuck
 *	for a LONG time, or if absolute order of processing is important.
 *
 * Setting PIDBITS to 4 handles process creation rates of up to 16/sec,
 * and gives a unique number overflow every 4 years or so.  This should
 * be good enough for most applications.
 */
#define	PIDBITS	4

	STATIC long
unique()
{
	return ( (time((long *)0)<<PIDBITS) | getpid()&((1<<PIDBITS)-1) )
	    & 0x7fffffff;
}
